-----------------------------------------------------------------------------------------------
-- Client Lua Script for XyrakBoostLite
-- Copyright (c) Xyrak. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "GameLib"
require "QuestLib"
require "ChatSystemLib"
 
-----------------------------------------------------------------------------------------------
-- XyrakBoostLite Module Definition
-----------------------------------------------------------------------------------------------
local XyrakBoostLite = {}
XyrakBoostLite.Options = {}		-- Holds per character settings

-----------------------------------------------------------------------------------------------
-- Localization
-----------------------------------------------------------------------------------------------
local L = Apollo.GetPackage("Gemini:Locale-1.0").tPackage:GetLocale("XyrakBoostLite", true)
local groupStringL = nil	-- holds GROUP string localized

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- Default addon settings for new characters
local tDefaultOptions = {
	EnableXyrakBoostLite = true,
	EnableAutoAccept = true,
	EnableAutoTurn = true,
	AutomateGroupQuests = false,
}
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function XyrakBoostLite:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    return o
end

function XyrakBoostLite:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 
-----------------------------------------------------------------------------------------------
-- XyrakBoostLite OnLoad
-----------------------------------------------------------------------------------------------
function XyrakBoostLite:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("XyrakBoostLite.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)

	-- Register events
	Apollo.RegisterEventHandler("Dialog_ShowState", "OnDialog_ShowState", self)

	-- Define variables
	groupStringL = L["GROUP"] -- Localize group string used by OnDialog_ShowState event
	self.version = XmlDoc.CreateFromFile("toc.xml"):ToTable().Version -- Get addon version from TOC file
end

-----------------------------------------------------------------------------------------------
-- XyrakBoostLite OnDocLoaded
-----------------------------------------------------------------------------------------------
function XyrakBoostLite:OnDocLoaded()
	-- Search for other addons that automate quest
	local whichAddon = self:CheckBlockerAddons()
	if whichAddon ~= nil then
		-- A blocker has been found... report and exit
		self:Print(L["Conflict with addon"] .. " " .. whichAddon)
		return
	end

	-- Register slash command
	Apollo.RegisterSlashCommand("xyrak", "OnSlashCommand", self)

	-- Load Default addon options if no saved options exist
	for optionName, optionValue in pairs(tDefaultOptions) do
		if XyrakBoostLite.Options[optionName] == nil then
			XyrakBoostLite.Options[optionName] = optionValue
		end
	end
end

-----------------------------------------------------------------------------------------------
-- Saving and Loading per Character Settings
-----------------------------------------------------------------------------------------------
function XyrakBoostLite:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	local tSavedData = {}
	for key,value in pairs(XyrakBoostLite.Options) do
		tSavedData[key] = XyrakBoostLite.Options[key]
	end
	return tSavedData
end

function XyrakBoostLite:OnRestore(eType, tSavedData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	for key,value in pairs(tSavedData) do
		XyrakBoostLite.Options[key] = value
	end
end

-----------------------------------------------------------------------------------------------
-- XyrakBoostLite Functions
-----------------------------------------------------------------------------------------------
-- Event that executes when talking to NPC and dialog shows
function XyrakBoostLite:OnDialog_ShowState(eState, queCurrent)
	-- Do not automate quests if XyrakBoostLite is disabled
	if not XyrakBoostLite.Options.EnableXyrakBoostLite then return end
	-- Proceed to analyze dialog entries
	local idQuest = 0
	if queCurrent then
		idQuest = queCurrent:GetId()
	end
	local tResponseList = DialogSys.GetResponses(idQuest)
	if not tResponseList or #tResponseList == 0 then return end
	-- Check responses and their types if settings allow it
	local tQuestResponses = {}
	for key, response in ipairs(tResponseList) do
		local responseType = response:GetType()
		local responseQuestId = response:GetQuestId()
		-- check for multiple responses
		if ((responseType == DialogResponse.DialogResponseType_QuestComplete or
		   responseType == DialogResponse.DialogResponseType_ViewQuestComplete) and
		   XyrakBoostLite.Options.EnableAutoTurn) or
		   ((responseType == DialogResponse.DialogResponseType_QuestAccept or
		   responseType == DialogResponse.DialogResponseType_ViewQuestAccept) and
		   XyrakBoostLite.Options.EnableAutoAccept) then
			if tQuestResponses[responseQuestId] then -- we already have a quest
				tQuestResponses[responseQuestId][ #tQuestResponses[responseQuestId] +1 ] = response
			else
				tQuestResponses[responseQuestId] = {}
				tQuestResponses[responseQuestId][1] = response
			end
		end
	end
	-- Selection logic
	for nQuestId, tResponses in pairs(tQuestResponses) do
		-- only process quests with 1 answer, because 2+ = gear choice
		if #tResponses == 1 then
			if tResponses[1]:GetText():match(groupStringL) then
				-- Group quests will only be handled if user opted to
				if XyrakBoostLite.Options.AutomateGroupQuests then
					tResponses[1]:Select()
				end
			else
				-- Solo quest, select as normal
				tResponses[1]:Select()
			end
		end
	end
end

-- On slash command "/xyrak"
function XyrakBoostLite:OnSlashCommand(strCmd,strArg)
	if strArg == "settings" then
		self:SettingsOpen() -- show settings window
	elseif strArg == "version" then
		self:Print("Version " .. self.version) -- print version
	else
		self:DisplayHelp() -- display command help
	end
end

---------------------------------------------------------------------------------------------------
-- SettingsForm related Functions
---------------------------------------------------------------------------------------------------
-- Enable optional controls in settings window
function XyrakBoostLite:OnXyrakBoostLiteCheck( wndHandler, wndControl, eMouseButton )
	self.Settings:FindChild("AutoAcceptQuestBtn"):Enable(true)
	self.Settings:FindChild("AutoAcceptQuestBtn"):SetBGColor("white")
	self.Settings:FindChild("AutoTurnQuestBtn"):Enable(true)
	self.Settings:FindChild("AutoTurnQuestBtn"):SetBGColor("white")
	self.Settings:FindChild("AutomateGroupQuestsBtn"):Enable(true)
	self.Settings:FindChild("AutomateGroupQuestsBtn"):SetBGColor("white")
end

-- Disable optional controls in settings window
function XyrakBoostLite:OnXyrakBoostLiteUncheck( wndHandler, wndControl, eMouseButton )
	self.Settings:FindChild("AutoAcceptQuestBtn"):Enable(false)
	self.Settings:FindChild("AutoAcceptQuestBtn"):SetBGColor("UI_AlphaPercent50")
	self.Settings:FindChild("AutoTurnQuestBtn"):Enable(false)
	self.Settings:FindChild("AutoTurnQuestBtn"):SetBGColor("UI_AlphaPercent50")
	self.Settings:FindChild("AutomateGroupQuestsBtn"):Enable(false)
	self.Settings:FindChild("AutomateGroupQuestsBtn"):SetBGColor("UI_AlphaPercent50")
end

-- Close the Settings window
function XyrakBoostLite:SettingsClose()
	-- Set settings to current control values
	XyrakBoostLite.Options.EnableXyrakBoostLite = self.Settings:FindChild("EnableXyrakBoostLiteBtn"):IsChecked()
	XyrakBoostLite.Options.EnableAutoAccept = self.Settings:FindChild("AutoAcceptQuestBtn"):IsChecked()
	XyrakBoostLite.Options.EnableAutoTurn = self.Settings:FindChild("AutoTurnQuestBtn"):IsChecked()
	XyrakBoostLite.Options.AutomateGroupQuests = self.Settings:FindChild("AutomateGroupQuestsBtn"):IsChecked()
	-- Close the window
	self.Settings:Show(false)
end

-- Display Settings Window
function XyrakBoostLite:SettingsOpen()
	-- Load Settings form if have not been done previously
	if self.xmlDoc:IsLoaded() and self.Settings == nil then
		-- Load it since is not defined
		self.Settings = Apollo.LoadForm(self.xmlDoc, "SettingsForm", nil, self)
		if self.Settings == nil then
			-- Loading failed, warn & exit
			Apollo.AddAddonErrorText(self, "SettingsForm:" .. L["Could not load it for some reason"] .. ".")
			return
		end
		-- Keep window hidden until we are done
		self.Settings:Show(false)
		-- Translate form labels
		self.Settings:FindChild("SettingsTitle"):SetText(L["XyrakBoostLite Settings"])
		self.Settings:FindChild("EnableXyrakBoostLiteBtn"):SetText(L["Enable XyrakBoostLite"])
		self.Settings:FindChild("AutoAcceptQuestBtn"):SetText(L["Auto Accept Quests"])
		self.Settings:FindChild("AutoTurnQuestBtn"):SetText(L["Auto Turn Quests"])
		self.Settings:FindChild("AutomateGroupQuestsBtn"):SetText(L["Automate Group Quests"])
	end
	-- Close a previous instance if available
	if self.Settings:IsShown() then self:SettingsClose() return end
	-- Set controls to current settings values
	self.Settings:FindChild("EnableXyrakBoostLiteBtn"):SetCheck(XyrakBoostLite.Options.EnableXyrakBoostLite)
	self.Settings:FindChild("AutoAcceptQuestBtn"):SetCheck(XyrakBoostLite.Options.EnableAutoAccept)
	self.Settings:FindChild("AutoTurnQuestBtn"):SetCheck(XyrakBoostLite.Options.EnableAutoTurn)
	self.Settings:FindChild("AutomateGroupQuestsBtn"):SetCheck(XyrakBoostLite.Options.AutomateGroupQuests)
	-- Display the window
	self.Settings:Show(true)
end

-----------------------------------------------------------------------------------------------
-- XyrakBoostLite Instance
-----------------------------------------------------------------------------------------------
local XyrakBoostLiteInst = XyrakBoostLite:new()
XyrakBoostLiteInst:Init()
